@@ -4,19 +4,20 @@ import datetime
 import numpy as np
 import numpy.typing as npt
 
-from timevec.builtin_math import time_elapsed_ratio
+from timevec.util import (
+    day_range,
+    month_range,
+    time_elapsed_ratio,
+    week_range,
+    year_range,
+)
 
 
 def year_vec(
     dt: datetime.datetime, *, dtype: npt.DTypeLike = np.float64
 ) -> npt.NDArray:
     """Represent the elapsed time in the year as a vector"""
-    begin_of_year = datetime.datetime(
-        dt.year, month=1, day=1, hour=0, minute=0, second=0
-    )
-    end_of_year = datetime.datetime(
-        dt.year + 1, month=1, day=1, hour=0, minute=0, second=0
-    )
+    begin_of_year, end_of_year = year_range(dt)
     rate = time_elapsed_ratio(
         begin=begin_of_year,
         end=end_of_year,
@@ -29,13 +30,7 @@ def month_vec(
     dt: datetime.datetime, *, dtype: npt.DTypeLike = np.float64
 ) -> npt.NDArray:
     """Represent the elapsed time in the month as a vector"""
-    begin_of_month = datetime.datetime(
-        year=dt.year, month=dt.month, day=1, hour=0, minute=0, second=0
-    )
-    _, last_day = calendar.monthrange(dt.year, dt.month)
-    end_of_month = datetime.datetime(
-        year=dt.year, month=dt.month, day=last_day, hour=0, minute=0, second=0
-    ) + datetime.timedelta(days=1)
+    begin_of_month, end_of_month = month_range(dt)
     rate = time_elapsed_ratio(
         begin=begin_of_month,
         end=end_of_month,
@@ -48,22 +43,7 @@ def week_vec(
     dt: datetime.datetime, *, dtype: npt.DTypeLike = np.float64
 ) -> npt.NDArray:
     """Represent the elapsed time in the week as a vector"""
-    begin_of_week = datetime.datetime.min.replace(
-        year=dt.year,
-        month=dt.month,
-        day=dt.day,
-        hour=0,
-        minute=0,
-        second=0,
-    ) - datetime.timedelta(days=dt.weekday())
-    end_of_week = datetime.datetime.min.replace(
-        year=dt.year,
-        month=dt.month,
-        day=dt.day,
-        hour=0,
-        minute=0,
-        second=0,
-    ) + datetime.timedelta(days=7 - dt.weekday())
+    begin_of_week, end_of_week = week_range(dt)
     rate = time_elapsed_ratio(
         begin=begin_of_week,
         end=end_of_week,
@@ -76,22 +56,7 @@ def day_vec(
     dt: datetime.datetime, *, dtype: npt.DTypeLike = np.float64
 ) -> npt.NDArray:
     """Represent the elapsed time in the day as a vector"""
-    begin_of_day = datetime.datetime.min.replace(
-        year=dt.year,
-        month=dt.month,
-        day=dt.day,
-        hour=0,
-        minute=0,
-        second=0,
-    )
-    end_of_day = datetime.datetime.min.replace(
-        year=dt.year,
-        month=dt.month,
-        day=dt.day,
-        hour=0,
-        minute=0,
-        second=0,
-    ) + datetime.timedelta(days=1)
+    begin_of_day, end_of_day = day_range(dt)
     rate = time_elapsed_ratio(
         begin=begin_of_day,
         end=end_of_day,
@@ -108,3 +73,24 @@ def ratio_to_vec(
     vec[0] = np.cos(2.0 * np.pi * ratio)
     vec[1] = np.sin(2.0 * np.pi * ratio)
     return vec
+
+
+def vec_to_ratio(arr: npt.NDArray) -> float:
+    """Convert a vector to a ratio"""
+    # atan2 returns a value in the range [-pi, pi]
+    # so we need to convert it to the range [0, 2*pi]
+    base = np.arctan2(arr[1], arr[0]) / (2.0 * np.pi)
+    return float(base if base >= 0.0 else base + 1.0)
+
+
+def datetime_from_vec(
+    year: int,
+    yv: npt.NDArray,
+    dv: npt.NDArray,
+) -> datetime.datetime:
+    position_in_year = vec_to_ratio(yv)
+    position_in_day = vec_to_ratio(dv)
+    d = int(position_in_year * (366.0 if calendar.isleap(year) else 365.0))
+    s = position_in_day * 86400.0
+    delta = datetime.timedelta(days=d, seconds=s)
+    return datetime.datetime(year, 1, 1, 0, 0, 0, 0) + delta
